@@ -104,7 +104,7 @@ class device_n900(deviceModule):
   def getDeviceName(self):
     return "Nokia N900"
 
-  def locationType(self):
+  def getLocationType(self):
     """modRana uses liblocation on N900"""
     return "liblocation"
 
