@@ -21,7 +21,7 @@ from __future__ import with_statement # for python 2.5
 from base_module import ranaModule
 from threading import Thread
 import threading
-import Queue
+import os
 import traceback
 import cairo
 import urllib2
@@ -32,7 +32,6 @@ import traceback
 import modrana_utils
 import rectangles
 import gobject
-from configobj import ConfigObj
 from tilenames import *
 
 import socket
@@ -48,48 +47,6 @@ socket.setdefaulttimeout(timeout)
 
 def getModule(m,d,i):
   return(MapTiles(m,d,i))
-
-maplayers = {}
-configVariables = {
-    'label':'label',
-    'url':'tiles',
-    'max_zoom':'maxZoom',
-    'min_zoom':'minZoom',
-    'type':'type',
-    'folder_prefix':'folderPrefix',
-    'coordinates':'coordinates',
-                  }
-mapConfigPath = 'map_config.conf'
-
-
-def allNeededIn(needed, dict):
-  for key in needed:
-    if key in dict:
-      continue
-    else:
-      return False
-  return True  
-
-try:
-  config = ConfigObj(mapConfigPath)
-  for layer in config:
-    if allNeededIn(configVariables.keys(), config[layer].keys()): # check if all neded keys are available
-      tempDict = {}
-      for var in configVariables:
-        tempDict[configVariables[var]] = config[layer][var]
-      tempDict['minZoom'] = int(tempDict['minZoom']) # convert strings to integers
-      tempDict['maxZoom'] = int(tempDict['maxZoom'])
-    else:
-      print "mapTiles: layer is badly defined/formated: %s" % layer
-
-
-    maplayers[layer] = tempDict
-
-except Exception, e:
-  print "mapTiles: loading map_config.conf failed: %s" % e
-
-
-
   
 class MapTiles(ranaModule):
   """Display map images"""
@@ -132,23 +89,38 @@ class MapTiles(ranaModule):
     self.waitingTile = self.images[1]['tileWaitingForDownloadSlot']
     self.lastThreadCleanupTimestamp=time.time()
     self.lastThreadCleanupInterval=2 # clean finished threads every 2 seconds
-    self.set('maplayers', maplayers) # export the maplyers definition for use by other modules
+
+    # local copy of the mapLayers dictionary
+    # TODO: don't forget to update this after implementing
+    #       map layer re/configuration at runtime
+    self.mapLayers = self.modrana.getMapLayers()
 
     self.mapViewModule = None
 
+    self.mapFolderPath = None
+
   def firstTime(self):
     self.mapViewModule = self.m.get('mapView', None)
-    self._updateScalingCB()
+    scale = self.get('mapScale', 1)
+    self._updateScalingCB('mapScale', scale, scale)
     self.modrana.watch('mapScale', self._updateScalingCB)
     self.modrana.watch('z', self._updateScalingCB)
 
-  def _updateScalingCB(self, key=None, oldValue=None, newValue=None):
+    # cache the map folder path
+    options = self.m.get('options', None)
+    if options:
+      self.mapFolderPath = options.getMapFolderPath()
+      print "mapTiles: map folder path: %s" % self.mapFolderPath
+
+  def _updateScalingCB(self, key='mapScale', oldValue=1, newValue=1):
     """
     as this only needs to be updated once on startup and then only
     when scaling settings change this callback driven method is used
     """
-
-    scale = int(self.get('mapScale', 1))
+    if key == 'mapScale':
+      scale = int(newValue)
+    else:
+      scale = int(self.get('mapScale', 1))
     if scale == 1: # this will be most of the time, so it is first
       z = int(self.get('z', 15))
     elif scale == 2: # tiles are scaled to 512*512 and represent tiles from zl-1
@@ -190,9 +162,9 @@ class MapTiles(ranaModule):
               for i in range(0, availableSlots):
                 if self.downloadRequestPool:
                   request = self.downloadRequestPool.pop() # download most recent requests first
-                  (name,x,y,z,layer,layerPrefix,layerType, filename, folder, timestamp) = request
+                  (name,x,y,z,layer,layerPrefix,layerType, filename, timestamp) = request
                   # start a new thread
-                  self.threads[name] = self.TileDownloader(name,x,y,z,layer,layerPrefix,layerType, filename, folder, self)
+                  self.threads[name] = self.TileDownloader(name,x,y,z,layer,layerPrefix,layerType, filename, self)
                   self.threads[name].daemon = True
                   self.threads[name].start()
                   # chenge the status tile to "Downloading..."
@@ -276,6 +248,7 @@ class MapTiles(ranaModule):
       """
       print("mapTiles: exception in idle loader\n", e)
       self.idleLoaderActive = False
+      traceback.print_exc()
       return False
 
   def loadSpecialTiles(self, specialTiles):
@@ -678,7 +651,7 @@ class MapTiles(ranaModule):
           return('OK')
     
     # seccond, is it in the disk cache?  (including ones recently-downloaded)
-    layerInfo = maplayers.get(layer, None)
+    layerInfo = self.mapLayers.get(layer, None)
     if(layerInfo == None): # is the layer info valid
       sprint("invalid layer")
       return('NOK')
@@ -702,7 +675,7 @@ class MapTiles(ranaModule):
         return('OK')
 
     # Image not found anywhere locally - resort to downloading it
-    filename = self.getTileFolderPath() + (self.getImagePath(x,y,z,layerPrefix, layerType))
+    filename = os.path.join(self._getTileFolderPath(), (self.getImagePath(x,y,z,layerPrefix, layerType)))
     sprint("image not found locally - trying to download")
 
     # Are we allowed to download it ? (network=='full')
@@ -712,9 +685,8 @@ class MapTiles(ranaModule):
       """the thread list condition is used to signalize to the download manager,
       that here is a new download request"""
       with self.threadlListCondition:
-        folder = self.getTileFolderPath() + self.getImageFolder(x, z, layerPrefix) # target folder
         timestamp = time.time()
-        request = (name,x,y,z,layer,layerPrefix,layerType, filename, folder, timestamp)
+        request = (name,x,y,z,layer,layerPrefix,layerType, filename, timestamp)
 
         with self.imagesLock: # display the "Waiting for download slot..." status tile
           waitingTile = self.waitingTile
@@ -843,26 +815,50 @@ class MapTiles(ranaModule):
   def getImagePath(self,x,y,z,prefix, extension):
     """Get a unique name for a tile image
     (suitable for use as part of filenames, dictionary keys, etc)"""
-    return("%s/%d/%d/%d.%s" % (prefix,z,x,y,extension))
+#    return("%s/%d/%d/%d.%s" % (prefix,z,x,y,extension))
+    return os.path.join(prefix, str(z), str(x), "%d.%s" % (y,extension) )
   
   def getImageFolder(self,x,z,prefix):
     """Get a unique name for a tile image
     (suitable for use as part of filenames, dictionary keys, etc)"""
     return("%s/%d/%d" % (prefix,z,x))
 
-  def getTileFolderPath(self):
+  def _getTileFolderPath(self):
     """helper function that returns path to the tile folder"""
-    return self.get('tileFolder', 'cache/images')
+    return self.mapFolderPath
 
   def imageY(self, z,extension):
     return (('%d.%s') % (z, extension))
 
-  def layers(self):
-    return(maplayers)
-
   def getTileUrl(self, x, y, z, layer):
-    """Wrapper, that makes it possible to use this function from other modules."""
-    return getTileUrl(x, y, z, layer)
+    """Return url for given tile coorindates and layer"""
+    layerDetails = self.mapLayers.get(layer, {})
+    if layerDetails == {}:
+      return None
+    coords = layerDetails['coordinates']
+    if coords == 'google':
+      url = '%s&x=%d&y=%d&z=%d' % (
+        layerDetails['tiles'],
+        x,y,z)
+    elif coords == 'quadtree': # handle Virtual Earth maps and satelite
+      quadKey = quadTree(x, y, z)
+      url = '%s%s?g=452' % ( #  dont know what the g argument is, maybe revision ? but its not optional
+                                layerDetails['tiles'], # get the url
+                                quadKey # get the tile identificator
+                                #layerDetails['type'] # get the correct extension (also works with png for
+                                )                    #  both maps and sat, but the original url is specific)
+    elif coords == 'yahoo': # handle Yaho maps, sat, overlay
+      y = ((2**(z-1) - 1) - y)
+      z = z + 1
+      url = '%s&x=%d&y=%d&z=%d&r=1' % ( # I have no idea what the r parameter is, r=0 or no r => grey square
+                                layerDetails['tiles'],
+                                x,y,z)
+    else: # OSM, Open Cycle, T@H -> equivalent to coords == osm
+      url = '%s%d/%d/%d.%s' % (
+        layerDetails['tiles'],
+        z,x,y,
+        layerDetails.get('type','png'))
+    return url
 
   def shutdown(self):
     # shutdown the worker/consumer threads
@@ -880,7 +876,7 @@ class MapTiles(ranaModule):
 
   class TileDownloader(Thread):
     """Downloads an image (in a thread)"""
-    def __init__(self,name, x,y,z,layer,layerName,layerType,filename, folder, callback):
+    def __init__(self,name, x,y,z,layer,layerName,layerType,filename, callback):
       Thread.__init__(self)
       self.name = name
       self.x = x
@@ -889,7 +885,6 @@ class MapTiles(ranaModule):
       self.layer = layer
       self.layerName = layerName
       self.layerType = layerType
-      self.folder = folder
       self.finished = 0
       self.filename = filename
       self.callback = callback
@@ -902,8 +897,7 @@ class MapTiles(ranaModule):
           self.y,
           self.z,
           self.layer,
-          self.filename,
-          self.folder)
+          self.filename)
         self.finished = 1
 
       # something is wrong with the server or url
@@ -948,7 +942,7 @@ class MapTiles(ranaModule):
             self.callback.threadlListCondition.notifyAll()
 
     def printErrorMessage(self, e):
-        url = getTileUrl(self.x,self.y,self.z,self.layer)
+        url = self.callback.getTileUrl(self.x,self.y,self.z,self.layer)
         print "mapTiles: download thread reports error"
         print "** we were doing this, when an exception occured:"
         print "** downloading tile: x:%d,y:%d,z:%d, layer:%s, filename:%s, url: %s" % ( \
@@ -962,9 +956,9 @@ class MapTiles(ranaModule):
         print "** traceback:\n"
         traceback.print_exc()
 
-    def downloadTile(self,name,x,y,z,layer,filename, folder):
+    def downloadTile(self,name,x,y,z,layer,filename):
       """Downloads an image"""
-      url = getTileUrl(x,y,z,layer)
+      url = self.callback.getTileUrl(x,y,z,layer)
 
       request = urllib2.urlopen(url)
 #      request = urllib.urlopen(url)
@@ -996,36 +990,7 @@ class MapTiles(ranaModule):
       # like this, currupted tiles should not get past the pixbuf loader and be stored
       m = self.callback.m.get('storeTiles', None)
       if m:
-        m.automaticStoreTile(content, self.layerName, self.z, self.x, self.y, self.layerType, filename, folder, fromThread = True)
-
-def getTileUrl(x,y,z,layer): #TODO: share this with mapData
-    """Return url for given tile coorindates and layer"""
-    layerDetails = maplayers.get(layer, None)
-    coords = layerDetails['coordinates']
-    if coords == 'google':
-      url = '%s&x=%d&y=%d&z=%d' % (
-        layerDetails['tiles'],
-        x,y,z)
-    elif coords == 'quadtree': # handle Virtual Earth maps and satelite
-      quadKey = quadTree(x, y, z)
-      url = '%s%s?g=452' % ( #  dont know what the g argument is, maybe revision ? but its not optional
-                                layerDetails['tiles'], # get the url
-                                quadKey # get the tile identificator
-                                #layerDetails['type'] # get the correct extension (also works with png for
-                                )                    #  both maps and sat, but the original url is specific)
-    elif coords == 'yahoo': # handle Yaho maps, sat, overlay
-      y = ((2**(z-1) - 1) - y)
-      z = z + 1
-      url = '%s&x=%d&y=%d&z=%d&r=1' % ( # I have no idea what the r parameter is, r=0 or no r => grey square
-                                layerDetails['tiles'],
-                                x,y,z)
-    else: # OSM, Open Cycle, T@H -> equivalent to coords == osm
-      url = '%s%d/%d/%d.%s' % (
-        layerDetails['tiles'],
-        z,x,y,
-        layerDetails.get('type','png'))
-    return url
-
+        m.automaticStoreTile(content, self.layerName, self.z, self.x, self.y, self.layerType, filename)
   
 # modified from: http://www.maptiler.org/google-maps-coordinates-tile-bounds-projection/globalmaptiles.py (GPL)
 def quadTree(tx, ty, zoom ):
