@@ -23,13 +23,15 @@ import os
 import modrana_utils
 
 def getModule(m,d,i):
-  return(options(m,d,i))
+  return(Options(m,d,i))
 
-class options(ranaModule):
+class Options(ranaModule):
   """Handle options"""
   def __init__(self, m, d, i):
     ranaModule.__init__(self, m, d, i)
     self.options = {}
+    """for fast searching defaults for corresponding keys"""
+    self.keyDefault = {}
 
     # profile folder
     self.profileFolderPath = self.modrana.getProfilePath()
@@ -41,6 +43,12 @@ class options(ranaModule):
     self.on = '<span color="green">ON</span>'
     self.off = '<span color="red">OFF</span>'
 
+    # items menu cache
+    self.itemMenus = {}
+
+    # item tools special menu name
+    self.keyStateListGroupID = None
+
   def getProfilePath(self):
     """return path to the profile folder"""
     # check if the path exists and create it if not
@@ -138,7 +146,7 @@ class options(ranaModule):
     """this metohod shuld be run only after menu module instance
     is vailable in self.menuModule and the options manu is clared,
     eq. has at least the escape button"""
-    # firs we add the category button to the options
+    # first we add the category button to the options
     id = self._getCategoryID(inId) # get a standardized id
     action="%sset:menu:%s%s" % (actionPrefix,id,actionSufix)
     self.menuModule.addItem('options', name, icon, action)
@@ -148,26 +156,70 @@ class options(ranaModule):
     return inId
 
   def _getGroupId(self, catId, id):
-    parrentId = self._getCategoryID(catId)
-    return "%s_opt_group_%s" % (parrentId, id)
+    parentId = self._getCategoryID(catId)
+    return "%s_opt_group_%s" % (parentId, id)
   
-  def addGroup(self,name,id,parrentId,icon,actionPrefix="",actionSufix=""):
+  def addGroup(self,name,id,parentId,icon,actionPrefix="",
+               actionSufix="", registerToMenu=True, backButtonAction=None):
     """this method ads a new (empty) options group to category specified by
     catId, as a convenience feature, the id of the new group is returned"""
-    catId = self._getCategoryID(parrentId)
-    id = self._getGroupId(parrentId, id) # get a standardized id
-    action="%sset:menu:%s%s" % (actionPrefix,id,actionSufix)
-    self.menuModule.addItem(catId, name, icon, action)
-    self.options[id] = ["set:menu:%s" % catId,0,[]]
+    catId = self._getCategoryID(parentId)
+    id = self._getGroupId(parentId, id) # get a standardized id
+    """
+    handle mepty parent ids - such ids can be valid because the menu switching is handled
+    handled entirelly by the pre and post actions
+    """
+    if not parentId:
+      action="%s%s" % (actionPrefix,actionSufix)
+    else:
+      action="%sset:menu:options#%s%s" % (actionPrefix,id,actionSufix)
+      
+    if registerToMenu: # add to options menu structure ?
+      self.menuModule.addItem(catId, name, icon, action)
+    if backButtonAction != None:
+      self.options[id] = [backButtonAction ,0,[]]
+    else:
+      self.options[id] = ["set:menu:%s" % catId,0,[]]
     return id
 
+  def setGroupParent(self, groupID, parentID):
+    """set the parent id of a given group id"""
+    if groupID in self.options:
+      self.options[groupID][0] = "set:menu:options#%s" % parentID
+    else:
+      print('options - set group parent: group not found: %s' % groupID)
+
+  def getGroupParent(self, groupID):
+    """set the parent id of a given group id"""
+    if groupID in self.options:
+      return self.options[groupID][0]
+    else:
+      print('options - get group parent: group not found: %s' % groupID)
+
+  def clearGroup(self, groupID):
+    """clear a given group from any options,
+    preserving its parent setting"""
+    self.options[groupID][1] = 0
+    self.options[groupID][2] = []
+
+
   def addBoolOption(self, title, variable, group, default=None, action=None):
     on = self.on
     off = self.off
+
     if action:
-      self.addOption(title,variable,((False,off,action),(True,on,action)),group,default)
+      states = ((False,off,action),(True,on,action))
     else:
-      self.addOption(title,variable,((False,off),(True,on)),group,default)
+      states = ((False,off),(True,on))
+    data = {"type":"toggle",
+               "states": states }
+    self.addOption(title,variable,data,group,default)
+
+  def addToggleOption(self, title, variable, choices, group, default=None):
+    data = {"type":"toggle",
+               "states": choices }
+    self.addOption(title,variable,data,group,default)
+
 
   def addEditOption(self, title, variable, group, label="Edit variable", description=None):
     choices = {"type":"showAndEditVariable",
@@ -176,14 +228,97 @@ class options(ranaModule):
               }
     self.addOption(title, variable, choices, group, None)
 
-  def addOption(self, title, variable, choices, group, default=None):
+  def addItemsOption(self, title, variable, items, group, default=None, fakeMode=None):
+    # the back action returns back to the group
+    backAction = "set:menu:options#%s" % group
+    # create and add the menu
+    menu = self.menuModule.getClearedMenu(backAction)
+    menuItems = [] # an ordered list of all the menu items
+    itemDict = {} # for easilly asigning keys to labels
+    if fakeMode != None:
+      mode = fakeMode
+    else:
+      mode = self.get('mode', 'car')
 
-    newOption = (title,variable, choices,group,default)
+    id = 1 # id 0 us the escape button
+    for key, name in items:
+      item = self.menuModule.generateItem("#%s" % name, "generic",
+      "setWithMode:%s:%s:%s|%s" % (mode, variable, key, backAction))
+
+      menuItems.append(item)
+      itemDict[key] = (name,id)
+      id = id + 1
+    # load all items to the menu
+    menu = self.menuModule.addItemsToThisMenu(menu, menuItems)
+    # store the menu in the menu module
+    """NOTE: for the returning back to the group to work correctly,
+    the menu is stored under a key combined from the variable and group names"""
+    storageKey = self._getItemsOptionStorageKey(group, variable, mode)
+
+    self.menuModule.addItemMenu(storageKey, menu, wideButtons=True)
+
+    # also store in the local options structure
+    choices = {"type":"selectOneItem",
+              'label':"",
+              'description':"",
+              'default':default,
+              'items':items,
+              'itemDict' : itemDict,
+              'storageKey' : storageKey}
+    if fakeMode != None:
+      choices['mode'] = fakeMode
+      print fakeMode
+    self.addOption(title, variable, choices, group, default)
+
+  def _getItemsOptionStorageKey(self, group, variable, mode):
+    """return menu name for the special item selection itemized menu
+    """
+    return "options1Item*%s*%s*%s" % (group, variable, mode)
+
+  def _highlightActiveItem(self, menu, variable):
+    """highlight currently active item in the item selection menu"""
+    # test if the key was initialized
+    if self.optionsKeyExists(variable):
+      text, icon, action, type, timedAction
+    else: # not initialized, no need to highlight anything
+      return menu
+
+  def addOption(self, title, variable, choices, group, default=None):
+    """add an option item"""
+  
+    """ add group name to choices,
+    this is needed for the item tools menu to know where to return"""
+    choices['groupName'] = group
+
+    newOption = [title,variable, choices,group,default]
     if self.options.has_key(group):
       self.options[group][2].append(newOption)
+      self.keyDefault[variable] = default
     else:
       print "options: group %s does not exist, call addGroup to create it first" % group
 
+  def addRawOption(self, optionData):
+    """add a raw option to options
+    NOTE: the options contains its group ID"""
+    (title,variable, choices,group,default) = optionData
+    """ as some options have have side effects when they are created,
+    we need to check option the type and replicate those effect as needed"""
+    type = choices['type']
+    choices = dict(choices)
+    if type == 'selectOneItem':
+      if 'mode' in choices:
+        fakeMode = choices['mode']
+      else:
+        fakeMode = None
+      items = choices['items']
+      self.addItemsOption(title, variable, items, group, default, fakeMode = fakeMode)
+    else: # no side effects, just add the raw option
+      if self.options.has_key(group):
+        self.options[group][2].append(optionData)
+        self.keyDefault[variable] = default
+      else:
+        print "options: group %s does not exist, can't add a raw option to it" % group
+
   def removeOption(self, categoryId, groupId, variable):
     """remova an option given by group and variable name"""
 
@@ -192,11 +327,28 @@ class options(ranaModule):
     if self.options.has_key(group):
       remove = lambda x:x[1]==variable
       self.options[group][2][:] = [x for x in self.options[group][2] if not remove(x)]
+      if variable in self.keyDefault:
+        del self.keyDefault[variable]
     else:
       print "options: group %s does not exist, so option with variable %s can not be removed" % (group,variable)
 
-#  def optionExists(self, categoryId, groupId):
-    
+  def getOption(self, groupID, index):
+    """get a options item from a given group by its index"""
+    if self.options.has_key(groupID):
+      try:
+        return self.options[groupID][2][index]
+      except IndexError:
+        print "options: group %s has no index %d, so this option can not be returned" % (groupID, index)
+        return False
+
+    else:
+      print "options: group %s does not exist, so option with index %d can not be returned" % (groupID, index)
+      return False
+
+  def getKeyDefault(self, key, default=None):
+    """get default value for a given key"""
+    return self.keyDefault.get(key, default)
+
   def firstTime(self):
     """Create a load of options.  You can add your own options in here,
     or alternatively create them at runtime from your module's firstTime()
@@ -209,105 +361,105 @@ class options(ranaModule):
     # shotcuts
     addCat = self.addCategory
     addGroup = self.addGroup
-    addOpt = self.addOption
+    addOpt = self.addToggleOption
     addBoolOpt = self.addBoolOption
+    addItems = self.addItemsOption
 
     # * the Map category *
     catMap = addCat("Map", "map", "map")
 
     # ** map layers
     group = addGroup("Map layers", "map_layers", catMap, "generic")
-    tiles = self.m.get("mapTiles", None)
-    if(tiles):
-      tileOptions = [("","None")]
-      layers = self.modrana.getMapLayers().items()
-      layers.sort()
-      for name,layer in layers:
-        tileOptions.append((name, layer.get('label',name)))
-      addOpt("Main map", "layer", tileOptions, group, "mapnik")
+    layers = self.modrana.getMapLayers()
+    layerNameKey = []
+    for key in layers.keys():
+      name = layers[key]['label']
+      layerNameKey.append( (key, name) )
+    layerNameKey.sort()
+    layerNameKey.append((None, "Empty layer"))
+    addItems("Main map", "layer", layerNameKey, group, "mapnik")
 
+    # ** Overlay
+    group = addGroup("Map overlay", "map_overlay", catMap, "generic")
+    addBoolOpt("Map as overlay", "overlay", group, False)
 
-      # ** Overlay
-      group = addGroup("Map overlay", "map_overlay", catMap, "generic")
-      addBoolOpt("Map as overlay", "overlay", group, False)
+    addItems("Main map", "layer", layerNameKey, group, "mapnik")
+    addItems("Background map", "layer2", layerNameKey, group, "osma")
 
-      addOpt("Main map", "layer", tileOptions, group, "mapnik")
-      addOpt("Background map", "layer2", tileOptions, group, "osma")
+    addOpt("Transparency ratio", "transpRatio",
+            [("0.25,1","overlay:25%"),
+            ("0.5,1","overlay:50%"),
+            ("0.75,1","overlay:75%"),
+            ("1,1","overlay:100%")],
+             group,
+             "0.5,1")
 
-      addOpt("Transparency ratio:", "transpRatio",
-              [("0.25,1","overlay:25%"),
-              ("0.5,1","overlay:50%"),
-              ("0.75,1","overlay:75%"),
-              ("1,1","overlay:100%")],
-               group,
-               "0.5,1")
+    # ** Rotation
+    group = addGroup("Rotation", "map_rotation", catMap, "generic")
+    addBoolOpt("Rotate map in direction of travel", "rotateMap", group, False)
 
-      # ** Rotation
-      group = addGroup("Rotation", "map_rotation", catMap, "generic")
-      addBoolOpt("Rotate map in direction of travel", "rotateMap", group, False)
+    # ** Scaling
+    group = addGroup("Scaling", "map_scaling", catMap, "generic")
+    addOpt("Map scale", "mapScale",
+                 [(1,"1X"),
+                  (2,"2X"),
+                  (4,"4X")],
+                   group,
+                   1)
 
-      # ** Scaling
-      group = addGroup("Scaling", "map_scaling", catMap, "generic")
-      addOpt("Map scale", "mapScale",
-                   [(1,"1X"),
-                    (2,"2X"),
-                    (4,"4X")],
-                     group,
-                     1)
+    # ** centering
+    group = addGroup("Centering", "centering", catMap, "generic")
+    addBoolOpt("Centre map", "centred", group, True)
 
-      # ** centering
-      group = addGroup("Centering", "centering", catMap, "generic")
-      addBoolOpt("Centre map", "centred", group, True)
+    addOpt("Centering shift", "posShiftDirection",
+      [("down","shift down"),
+       ("up","shift up"),
+       ("left","shift left"),
+       ("right","shift right"),
+       (False,"don't shift")],
+       group,
+       "down")
 
-      addOpt("Centering shift", "posShiftDirection",
-        [("down","shift down"),
-         ("up","shift up"),
-         ("left","shift left"),
-         ("right","shift right"),
-         (False,"don't shift")],
-         group,
-         "down")
+    addOpt("Centering shift amount", "posShiftAmount",
+      [(0.25,"25%"),
+       (0.5,"50%"),
+       (0.75,"75%"),
+       (1.0,"edge of the screen")],
+       group,
+       0.75)
 
-      addOpt("Centering shift amount", "posShiftAmount",
-        [(0.25,"25%"),
-         (0.5,"50%"),
-         (0.75,"75%"),
-         (1.0,"edge of the screen")],
-         group,
-         0.75)
+    changedMsg = "mapView:centeringDisableTresholdChanged"
+    addOpt("Disable by dragging", "centeringDisableTreshold",
+      [(2048,"normal drag - <i>default</i>",changedMsg),
+       (15000,"long drag",changedMsg),
+       (40000,"realy long drag",changedMsg),
+       (80000,"extremely long drag",changedMsg),
+       (False,self.off,changedMsg)],
+       group,
+       2048)
 
-      changedMsg = "mapView:centeringDisableTresholdChanged"
-      addOpt("Disable by dragging", "centeringDisableTreshold",
-        [(2048,"normal drag - <i>default</i>",changedMsg),
-         (15000,"long drag",changedMsg),
-         (40000,"realy long drag",changedMsg),
-         (80000,"extremely long drag",changedMsg),
-         (False,self.off,changedMsg)],
-         group,
-         2048)
+    # ** dragging
+    group = addGroup("Dragging", "dragging", catMap, "generic")
+    # check if we are on a powerful device or not and set the default accordingly
+    if self.dmod.simpleMapDragging():
+      defaultMode = "staticMapDrag"
+    else:
+      defaultMode = "default"
 
-      # ** dragging
-      group = addGroup("Dragging", "dragging", catMap, "generic")
-      # check if we are on a powerful device or not and set the default accordingly
-      if self.dmod.simpleMapDragging():
-        defaultMode = "staticMapDrag"
-      else:
-        defaultMode = "default"
+    addOpt("Map dragging", "mapDraggingMode",
+      [("default","full redraw - <i>default</i>","mapView:dragModeChanged"),
+       ("staticMapDrag","drag visible map - <i>fastest</i>", "mapView:dragModeChanged")],
+       group,
+       defaultMode)
 
-      addOpt("Map dragging", "mapDraggingMode",
-        [("default","full redraw - <i>default</i>","mapView:dragModeChanged"),
-         ("staticMapDrag","drag visible map - <i>fastest</i>", "mapView:dragModeChanged")],
-         group,
-         defaultMode)
-
-      # ** tile storage
-      group = addGroup("Tile storage", "tile_storage", catMap, "generic")
-      addOpt("Tile storage", "tileStorageType",
-                   [('files',"files (default, more space used)"),
-                    ('sqlite',"sqlite (new, less space used)")],
-                     group,
-                     'files')
-      addBoolOpt("Store downloaded tiles", "storeDownloadedTiles", group, True)
+    # ** tile storage
+    group = addGroup("Tile storage", "tile_storage", catMap, "generic")
+    addOpt("Tile storage", "tileStorageType",
+                 [('files',"files (default, more space used)"),
+                  ('sqlite',"sqlite (new, less space used)")],
+                   group,
+                   'files')
+    addBoolOpt("Store downloaded tiles", "storeDownloadedTiles", group, True)
 
     # * the view category *
     catView = addCat("View", "view", "view")
@@ -534,7 +686,7 @@ class options(ranaModule):
                           ('tr tr', 'Turkish'),
                           ('vi vi', 'Vietnamese')]
 
-    addOpt("Language for directions", 'directionsLanguage',
+    addItems("Language for directions", 'directionsLanguage',
          directionsLanguages,
          group,
          "en en") # TODO: use locale for default language ?
@@ -600,7 +752,7 @@ class options(ranaModule):
     # ** GPS
     group = addGroup("GPS", "gps", catLocation, "generic")
     addBoolOpt("GPS", "GPSEnabled", group, True, "location:checkGPSEnabled")
-    if self.dmod.locationType() == 'gpsd':
+    if self.dmod.getLocationType() == 'gpsd':
       knots = "knots per second"
       meters = "meters per second"
       if self.device == 'neo':
@@ -652,7 +804,9 @@ class options(ranaModule):
     if self.get('voiceParameters', None) == "manual":
       self._updateVoiceManual('add')
 
-       
+    # *** special group for a list per mode item states ***
+    self.keyStateListGroupID = addGroup("specialTools", 'specialGroup', 'specialParent',
+    "generic", registerToMenu=False, backButtonAction="set:menu:optionsItemTools")    
 
 #    addOpt("Network", "threadedDownload",
 ##      [("off","No use of network"),
@@ -705,7 +859,12 @@ class options(ranaModule):
     addBoolOpt("Print Turn-By-Turn triggers", "debugTbT", group, False)
     # ** gps
     group = self.addGroup("GPS", "gps", catDebug, "generic")
-    addBoolOpt("Show N900 GPS-fix", "n900GPSDebug", group, False)
+    # only show relevant
+    locationType = self.dmod.getLocationType()
+    if locationType == 'liblocation':
+      addBoolOpt("Show N900 GPS-fix", "n900GPSDebug", group, False)
+    if locationType == 'gpsd':
+      addBoolOpt("Show gpsd debug output", "gpsdDebugVerbose", group, False, "location:gpsdCheckVerboseDebugEnabled")
 
 
 
@@ -736,11 +895,38 @@ class options(ranaModule):
       print('options: error while filtering options\nsome nonpersistent keys might have been left in\nNOTE: keys should be strings of length>=1\n', e)
       return self.d
 
+  def _reloadKeyStateList(self, groupID, index, key):
+    """reload the key state list to represent currently selected option"""
+
+    # clear the group
+    self.clearGroup(self.keyStateListGroupID)
+    """ for each mode show the current key state"""
+    modes = self.modrana.getModes().keys()
+    modes.sort()
+
+    # get data for the given option
+    optionData = self.getOption(groupID, index)
+
+
+    # modify the title
+    for (mode) in modes:
+      optionD = list(optionData) # make a copy
+      # modify the choices dictionary
+      # NOTE: somehow, it is needed to do a copy not just to modify it in the option
+      # or else the mode value is for all options added the same
+      d = dict(optionD[2])
+      d['noToolsIcon'] = True # disable the tools icon
+      d['mode'] = mode # force a mode
+      optionD[2] = d
+      optionD[3] = self.keyStateListGroupID # set the group to the state list
+      self.addRawOption(optionD)
+
   def save(self):
     print "options: saving options"
     try:
       f = open(self.getOptionsFilePath(), "w")
       # remove keys marked as nonpersistent
+      self.d['keyModifiers'] = self.modrana.keyModifiers
       d = self._removeNonPersistent(self.d)
       marshal.dump(d, f)
       f.close()
@@ -788,6 +974,47 @@ class options(ranaModule):
         self.options[menuName][1] = newIndex
     elif(message == "save"):
       self.save()
+
+    elif type == 'ml' and message == "go2ItemToolsMenu":
+      (groupID, index, key) = args
+      index = int(index)
+      # reload the tools menu
+      menus = self.m.get('menu', None)
+      if menus:
+        menuName = 'optionsItemTools'
+        reset='ms:options:resetKey:%s' % key
+        notify = "ml:notification:m:Item has been reset to default;3"
+        resetAction = "%s|%s|set:menu:options#%s" % (reset, notify, groupID)
+        menus.clearMenu(menuName, 'set:menu:options#%s' % groupID)
+        menus.addItem(menuName, 'state list#per mode', 'generic',
+                      'ml:options:go2ItemStateListMenu:%s;%d;%s' % (groupID, index, key)
+                      )
+        menus.addItem(menuName, 'default#reset to', 'generic', resetAction)
+        self.set('menu', menuName)
+    elif type == 'ml' and message == "go2ItemStateListMenu":
+      (groupID, index, key) = args
+      index = int(index)
+      """reload the option key state list for the given key"""
+      self._reloadKeyStateList(groupID, index, key)
+      # go to the menu
+      self.set('menu', 'options#%s' % self.keyStateListGroupID)
+
+    elif type == 'ms' and message == 'resetKey':
+      """ reset a given options item to default, including any key modifiers"""
+      key = args
+      self.modrana.purgeKey(key)
+      default = self.getKeyDefault(key)
+      self.set(key, default)
+      
+    elif type == 'ml' and message == 'addKeyModifier':
+      """make the value of a key mode specific"""
+      (key, mode) = args
+      self.modrana.addKeyModifier(key, mode=mode)
+    elif type == 'ml' and message == 'removeKeyModifier':
+      (key, mode) = args
+      """make the value of a key mode unspecific"""
+      self.modrana.removeKeyModifier(key, mode=mode)
+
     elif type=="ms" and message == "espeakParams":
       # switch between espeak parameter modes
       if args == "manual":
@@ -803,6 +1030,15 @@ class options(ranaModule):
         key = "editVariable_%s" % variable
         entry.entryBox(self,key, label, initialText, description)
 
+    #messages to toggles a key to be mode un/specific
+    #Mode specific keys:
+    #this means that the options key can have a different value
+    #depending on the current mode, thus enabling better cutomization"""
+    elif type == "ms" and message == "makeKeyModeSpecific":
+      self.modrana.addKeyModifier(args)
+    elif type == "ms" and message == "makeKeyModeUnSpecific":
+      self.modrana.removeKeyModifier(args)
+
   def _updateVoiceManual(self, action):
     """add or remove custom voce parameters option items"""
 
@@ -813,7 +1049,7 @@ class options(ranaModule):
       self.addEditOption("Edit voice string", "voiceString", groupId, "Edit voice string", description=description)
 
       message = "ms:voice:resetStringToDefault:espeak"
-      self.addOption("Reset voice string with <b>Espeak</b> default", "placeholder",
+      self.addToggleOption("Reset voice string with <b>Espeak</b> default", "placeholder",
       [("foo","<i>click to use this default</i>",message)],
        groupId,
        "foo")
@@ -828,19 +1064,10 @@ class options(ranaModule):
       print "editing variable: %s with: %s" % (variable, result)
       self.set(variable,result)
 
-  def drawMenu(self, cr, menuName):
-    """Draw menus"""
-    if(menuName[0:5] != "opt_c"):
-      return
-    if(not self.options.has_key(menuName)):
-      return
-    
-    # Find the screen
-    if not self.d.has_key('viewport'):
-      return
-   
-    if(self.menuModule):
-      
+  def drawMenu(self, cr, menuName, args=None):
+    # custom options list drawing
+    clickHandler = self.m.get('clickHandler', None)
+    if self.menuModule and clickHandler:
       # elements allocation
       (e1,e2,e3,e4,alloc) = self.menuModule.threePlusOneMenuCoords()
       (x1,y1) = e1
@@ -866,32 +1093,79 @@ class options(ranaModule):
         numItems = len(options)
         cAction = None
         if(0 <= index < numItems):
-          (title,variable,choices,category,default) = options[index]
+          (title,variable,choices,group,default) = options[index]
           # What's it set to currently?
-          value = self.get(variable, None)
+          if 'mode' in choices:
+            mode = choices['mode']
+          else:
+            mode = self.get('mode', 'car')
+          value = self.get(variable, None, mode=mode)
+
+          """ if the key has a modifier in this mode,
+          append the mode label to the title"""
+
+          if 'mode' in choices:
+            # this currently means we are in the option state list
+            if self.get('mode', 'car') == choices['mode']:
+              # current mode
+              title = "%s: <small><sup><b>[%s]</b></sup></small>" % (
+              title, self.modrana.getModeLabel(mode))
+            else:
+              title = "%s: <small><sup>[%s]</sup></small>" % (
+              title, self.modrana.getModeLabel(mode))
+
+          else:
+            # nromal option display
+            if self.modrana.hasKeyModifierInMode(variable, mode):
+              title = "%s: <small><sup><b>[%s]</b></sup></small>" % (
+              title, self.modrana.getModeLabel(mode))
+            else:
+              title = "%s:" % title
 
           # Lookup the description of the currently-selected choice.
           # (if any, use str(value) if it doesn't match any defined options)
           # Also lookup the _next_ choice in the list, because that's what
           # we will set the option to if it's clicked
 
-          if type(choices) is dict:
-            optionType = choices["type"]
+          optionType = choices["type"]
+
+          if optionType == "showAndEditVariable":
             label = choices["label"]
             description = choices["description"]
+            # show and edit the exact value of a variable manually
+            valueDescription = self.get(variable, "variable is not set yet")
+            valueDescription = "<tt><b>%s</b></tt>" % valueDescription
+            payload = "%s;%s;%s" % (variable, label, description)
+            onClick = "ml:options:editVariable:%s|set:needRedraw:True" % payload
 
-            if optionType == "showAndEditVariable":
-              valueDescription = self.get(variable, "variable is not set yet")
-              valueDescription = "<tt><b>%s</b></tt>" % valueDescription
-              payload = "%s;%s;%s" % (variable, label, description)
-              onClick = "ml:options:editVariable:%s|set:needRedraw:True" % payload
+          elif optionType == "selectOneItem":
+            #show multiple items and make it possible to select one of them
 
-          else: # toggle button
+            # get current value
+            default = choices['default']
+            value = self.get(variable, default, mode=mode)
+            # show label for the given value
+            valueDescription, highlightId = choices['itemDict'].get(value, (value,None))
+            """if no description is found, just display the value"""
+            valueDescription = "<tt><b>%s</b></tt>" % valueDescription
 
-            nextChoice = choices[0]
+            #assure highlighting
+            if highlightId != None:
+              """ run an action before switching to the next menu that
+              assures that items in the next menu are properly highlighted
+              according to the state of the corresponding variable"""
+              pre = "ml:menu:highlightItem:%s;%d|" % (choices['storageKey'], highlightId)
+            else:
+              pre = "" # no id that needs highlighting found
+
+            onClick = "%sset:menu:options1Item*%s*%s*%s" % (pre, group, variable, mode)
+
+          elif optionType == 'toggle':
+            states = choices['states']
+            nextChoice = states[0]
             valueDescription = str(value)
             useNext = False
-            for c in choices:
+            for c in states:
               (cVal, cName) = (c[0],c[1])
               if(useNext):
                 nextChoice = c
@@ -904,16 +1178,25 @@ class options(ranaModule):
 
             # What should happen if this option is clicked -
             # set the associated option to the next value in sequence
-            onClick = "set:%s:%s" % (variable, str(nextChoice[0]))
+            onClick = "setWithMode:%s:%s:%s" % (mode, variable, str(nextChoice[0]))
             if cAction:
               onClick += "|%s" % cAction
             onClick += "|options:save"
             onClick += "|set:needRedraw:1"
 
+
           y = y4 + (row) * dy
-          w = w1 - (x4-x1)
-          
-          # Draw background and make clickable
+          if w1 > h1: # landscape
+            dx = (x4-x1)
+            w = w1 - dx
+          else: # portrait
+            dx = (x2)
+            w = w1
+
+          smallButtonW = dx/2.0
+          smallButtonH = dy/2.0
+
+          # Draw the option button and make it clickable
           self.menuModule.drawButton(cr,
             x4,
             y,
@@ -921,18 +1204,51 @@ class options(ranaModule):
             dy,
             None,
             "generic", # background for a 3x1 icon
-            onClick)
+            "")
+          # due to the button on the righ, register a slightly smaller area
+          clickHandler.registerXYWH(x4, y, w-smallButtonW, dy, onClick)
+            
+          # draw mode specific combined toggle & indicator
+          if self.modrana.hasKeyModifierInMode(variable, mode):
+            toggleText = '<span color="green">ON</span>#per Mode'
+            modeSpecToggleAction = "ml:options:removeKeyModifier:%s;%s" % (variable, mode)
+          else:
+            toggleText = "OFF#per Mode"
+            modeSpecToggleAction = "ml:options:addKeyModifier:%s;%s" % (variable, mode)
+
+          self.menuModule.drawButton(cr,
+            x4+w-smallButtonW,
+            y,
+            smallButtonW,
+            smallButtonH,
+            toggleText,
+            "generic",
+            modeSpecToggleAction)
+
+          groupName = choices['groupName']
+          if 'noToolsIcon' not in choices:
+            # draw tools button
+            self.menuModule.drawButton(cr,
+              x4+w-smallButtonW,
+              y+smallButtonH,
+              smallButtonW,
+              smallButtonH,
+              None,
+              "tools", # tools icon
+              "ml:options:go2ItemToolsMenu:%s;%d;%s" % (groupName, index, variable))
+
 
           border = 20
 
           # 1st line: option name
-          self.menuModule.showText(cr, title+":", x4+border, y+border, w-2*border)
+          self.menuModule.showText(cr, title, x4+border, y+border, w*0.95 - smallButtonW - border)
 
           # 2nd line: current value
-          self.menuModule.showText(cr, valueDescription, x4 + 0.15 * w, y + 0.6 * dy, w * 0.85 - border)
+          self.menuModule.showText(cr, valueDescription, x4 + 0.15 * w, y + 0.6 * dy, w*0.85 - smallButtonW - border)
 
           # in corner: row number
-          self.menuModule.showText(cr, "%d/%d" % (index+1, numItems), x4+0.85*w, y+3*border, w * 0.15 - border, 20)
+          indexX = x4+w*0.90-smallButtonW
+          self.menuModule.showText(cr, "%d/%d" % (index+1, numItems), indexX, y+dy*0.07, w * 0.10 - border, 20)
 
 #  def checkProfilePath(self):
 #    """check if the profile folder exists, try to create it if not"""
